@@ -35,7 +35,8 @@ static const std::string option_file_header =
     "#\n"
     "\n";
 
-Status PersistRocksDBOptions(const DBOptions& db_opt,
+Status PersistRocksDBOptions(const WriteOptions& write_options,
+                             const DBOptions& db_opt,
                              const std::vector<std::string>& cf_names,
                              const std::vector<ColumnFamilyOptions>& cf_opts,
                              const std::string& file_name, FileSystem* fs) {
@@ -48,11 +49,12 @@ Status PersistRocksDBOptions(const DBOptions& db_opt,
   if (db_opt.log_readahead_size > 0) {
     config_options.file_readahead_size = db_opt.log_readahead_size;
   }
-  return PersistRocksDBOptions(config_options, db_opt, cf_names, cf_opts,
-                               file_name, fs);
+  return PersistRocksDBOptions(write_options, config_options, db_opt, cf_names,
+                               cf_opts, file_name, fs);
 }
 
-Status PersistRocksDBOptions(const ConfigOptions& config_options_in,
+Status PersistRocksDBOptions(const WriteOptions& write_options,
+                             const ConfigOptions& config_options_in,
                              const DBOptions& db_opt,
                              const std::vector<std::string>& cf_names,
                              const std::vector<ColumnFamilyOptions>& cf_opts,
@@ -67,8 +69,9 @@ Status PersistRocksDBOptions(const ConfigOptions& config_options_in,
   }
   std::unique_ptr<FSWritableFile> wf;
 
-  Status s =
-      fs->NewWritableFile(file_name, FileOptions(), &wf, nullptr);
+  FileOptions file_options;
+  file_options.temperature = db_opt.metadata_write_temperature;
+  Status s = fs->NewWritableFile(file_name, file_options, &wf, nullptr);
   if (!s.ok()) {
     return s;
   }
@@ -79,62 +82,70 @@ Status PersistRocksDBOptions(const ConfigOptions& config_options_in,
 
   std::string options_file_content;
 
-  s = writable->Append(
-      option_file_header + "[" + opt_section_titles[kOptionSectionVersion] +
-      "]\n"
-      "  rocksdb_version=" +
-      std::to_string(ROCKSDB_MAJOR) + "." + std::to_string(ROCKSDB_MINOR) +
-      "." + std::to_string(ROCKSDB_PATCH) + "\n");
+  IOOptions opts;
+  s = WritableFileWriter::PrepareIOOptions(write_options, opts);
   if (s.ok()) {
-    s = writable->Append(
-        "  options_file_version=" + std::to_string(ROCKSDB_OPTION_FILE_MAJOR) +
-        "." + std::to_string(ROCKSDB_OPTION_FILE_MINOR) + "\n");
+    s = writable->Append(opts, option_file_header + "[" +
+                                   opt_section_titles[kOptionSectionVersion] +
+                                   "]\n"
+                                   "  rocksdb_version=" +
+                                   std::to_string(ROCKSDB_MAJOR) + "." +
+                                   std::to_string(ROCKSDB_MINOR) + "." +
+                                   std::to_string(ROCKSDB_PATCH) + "\n");
   }
   if (s.ok()) {
-    s = writable->Append("\n[" + opt_section_titles[kOptionSectionDBOptions] +
-                         "]\n  ");
+    s = writable->Append(
+        opts,
+        "  options_file_version=" + std::to_string(ROCKSDB_OPTION_FILE_MAJOR) +
+            "." + std::to_string(ROCKSDB_OPTION_FILE_MINOR) + "\n");
+  }
+  if (s.ok()) {
+    s = writable->Append(
+        opts, "\n[" + opt_section_titles[kOptionSectionDBOptions] + "]\n  ");
   }
 
   if (s.ok()) {
     s = GetStringFromDBOptions(config_options, db_opt, &options_file_content);
   }
   if (s.ok()) {
-    s = writable->Append(options_file_content + "\n");
+    s = writable->Append(opts, options_file_content + "\n");
   }
 
   for (size_t i = 0; s.ok() && i < cf_opts.size(); ++i) {
     // CFOptions section
-    s = writable->Append("\n[" + opt_section_titles[kOptionSectionCFOptions] +
-                         " \"" + EscapeOptionString(cf_names[i]) + "\"]\n  ");
+    s = writable->Append(
+        opts, "\n[" + opt_section_titles[kOptionSectionCFOptions] + " \"" +
+                  EscapeOptionString(cf_names[i]) + "\"]\n  ");
     if (s.ok()) {
       s = GetStringFromColumnFamilyOptions(config_options, cf_opts[i],
                                            &options_file_content);
     }
     if (s.ok()) {
-      s = writable->Append(options_file_content + "\n");
+      s = writable->Append(opts, options_file_content + "\n");
     }
     // TableOptions section
     auto* tf = cf_opts[i].table_factory.get();
     if (tf != nullptr) {
       if (s.ok()) {
         s = writable->Append(
-            "[" + opt_section_titles[kOptionSectionTableOptions] + tf->Name() +
-            " \"" + EscapeOptionString(cf_names[i]) + "\"]\n  ");
+            opts, "[" + opt_section_titles[kOptionSectionTableOptions] +
+                      tf->Name() + " \"" + EscapeOptionString(cf_names[i]) +
+                      "\"]\n  ");
       }
       if (s.ok()) {
         options_file_content.clear();
         s = tf->GetOptionString(config_options, &options_file_content);
       }
       if (s.ok()) {
-        s = writable->Append(options_file_content + "\n");
+        s = writable->Append(opts, options_file_content + "\n");
       }
     }
   }
   if (s.ok()) {
-    s = writable->Sync(true /* use_fsync */);
+    s = writable->Sync(opts, true /* use_fsync */);
   }
   if (s.ok()) {
-    s = writable->Close();
+    s = writable->Close(opts);
   }
   TEST_SYNC_POINT("PersistRocksDBOptions:written");
   if (s.ok()) {
@@ -179,8 +190,8 @@ Status RocksDBOptionsParser::ParseSection(OptionSection* section,
   *section = kOptionSectionUnknown;
   // A section is of the form [<SectionName> "<SectionArg>"], where
   // "<SectionArg>" is optional.
-  size_t arg_start_pos = line.find("\"");
-  size_t arg_end_pos = line.rfind("\"");
+  size_t arg_start_pos = line.find('\"');
+  size_t arg_end_pos = line.rfind('\"');
   // The following if-then check tries to identify whether the input
   // section has the optional section argument.
   if (arg_start_pos != std::string::npos && arg_start_pos != arg_end_pos) {
@@ -224,7 +235,7 @@ Status RocksDBOptionsParser::ParseStatement(std::string* name,
                                             std::string* value,
                                             const std::string& line,
                                             const int line_num) {
-  size_t eq_pos = line.find("=");
+  size_t eq_pos = line.find('=');
   if (eq_pos == std::string::npos) {
     return InvalidArgument(line_num, "A valid statement must have a '='.");
   }
@@ -733,4 +744,3 @@ Status RocksDBOptionsParser::VerifyTableFactory(
   return Status::OK();
 }
 }  // namespace ROCKSDB_NAMESPACE
-
