@@ -52,27 +52,42 @@ class FilterBlockBuilder {
 
   virtual ~FilterBlockBuilder() {}
 
-  virtual void Add(
-      const Slice& key_without_ts) = 0;  // Add a key to current filter
+  // Add a key to current filter.
+  virtual void Add(const Slice& key_without_ts) = 0;
+  // A potentially more efficient version of Add(), though you cannot go back
+  // to Add() after using AddWithPrevKey() on a FilterBlockBuilder.
+  // prev_key_without_ts should be the empty Slice for the first key added
+  // (regardless of comparator; e.g. for bootstrapping delta encoding).
+  // More detail: The previous key is used when filters are key-range
+  // partitioned, and the PartitionedFilterBlockBuilder doesn't need to buffer
+  // the previous key when it is provided by calling this function.
+  virtual void AddWithPrevKey(const Slice& key_without_ts,
+                              const Slice& /*prev_key_without_ts*/) = 0;
+
   virtual bool IsEmpty() const = 0;      // Empty == none added
   // For reporting stats on how many entries the builder considered unique
   virtual size_t EstimateEntriesAdded() = 0;
-  Slice Finish() {  // Generate Filter
-    const BlockHandle empty_handle;
-    Status dont_care_status;
-    auto ret = Finish(empty_handle, &dont_care_status);
-    assert(dont_care_status.ok());
-    return ret;
-  }
-  // If filter_data is not nullptr, Finish() may transfer ownership of
+
+  // When using AddWithPrevKey, this must be called before Finish(). (May also
+  // be called without AddWithPrevKey, but prev_key_without_ts must be
+  // accurate regardless.)
+  virtual void PrevKeyBeforeFinish(const Slice& /*prev_key_without_ts*/) {}
+
+  // Generate a filter block. Returns OK if finished, or Incomplete if more
+  // filters are needed (partitioned filter). In the latter case, subsequent
+  // calls require the BlockHandle of the most recently generated and written
+  // filter, in last_partition_block_handle.
+  //
+  // If filter_owner is not nullptr, Finish() may transfer ownership of
   // underlying filter data to the caller,  so that it can be freed as soon as
   // possible. BlockBasedFilterBlock will ignore this parameter.
   //
-  virtual Slice Finish(
-      const BlockHandle& tmp /* only used in PartitionedFilterBlock as
-                                last_partition_block_handle */
-      ,
-      Status* status, std::unique_ptr<const char[]>* filter_data = nullptr) = 0;
+  // For either OK or Incomplete, *filter is set to point to the next filter
+  // bytes, which survive until either this is destroyed, *filter_owner is
+  // destroyed, or next call to Finish.
+  virtual Status Finish(
+      const BlockHandle& last_partition_block_handle, Slice* filter,
+      std::unique_ptr<const char[]>* filter_owner = nullptr) = 0;
 
   // This is called when finishes using the FilterBitsBuilder
   // in order to release memory usage and cache charge
@@ -85,6 +100,16 @@ class FilterBlockBuilder {
   virtual Status MaybePostVerifyFilter(const Slice& /* filter_content */) {
     return Status::OK();
   }
+
+#ifndef NDEBUG
+  Slice TEST_Finish() {  // Generate Filter
+    const BlockHandle empty_handle;
+    Slice filter;
+    Status status = Finish(empty_handle, &filter);
+    assert(status.ok());
+    return filter;
+  }
+#endif  // NDEBUG
 };
 
 // A FilterBlockReader is used to parse filter from SST table.
@@ -100,39 +125,34 @@ class FilterBlockReader {
   FilterBlockReader& operator=(const FilterBlockReader&) = delete;
 
   /**
-   * If no_io is set, then it returns true if it cannot answer the query without
-   * reading data from disk. This is used in PartitionedFilterBlockReader to
-   * avoid reading partitions that are not in block cache already
-   *
    * Normally filters are built on only the user keys and the InternalKey is not
    * needed for a query. The index in PartitionedFilterBlockReader however is
    * built upon InternalKey and must be provided via const_ikey_ptr when running
    * queries.
    */
-  virtual bool KeyMayMatch(const Slice& key, const bool no_io,
-                           const Slice* const const_ikey_ptr,
+  virtual bool KeyMayMatch(const Slice& key, const Slice* const const_ikey_ptr,
                            GetContext* get_context,
                            BlockCacheLookupContext* lookup_context,
                            const ReadOptions& read_options) = 0;
 
-  virtual void KeysMayMatch(MultiGetRange* range, const bool no_io,
+  virtual void KeysMayMatch(MultiGetRange* range,
                             BlockCacheLookupContext* lookup_context,
                             const ReadOptions& read_options) {
     for (auto iter = range->begin(); iter != range->end(); ++iter) {
       const Slice ukey_without_ts = iter->ukey_without_ts;
       const Slice ikey = iter->ikey;
       GetContext* const get_context = iter->get_context;
-      if (!KeyMayMatch(ukey_without_ts, no_io, &ikey, get_context,
-                       lookup_context, read_options)) {
+      if (!KeyMayMatch(ukey_without_ts, &ikey, get_context, lookup_context,
+                       read_options)) {
         range->SkipKey(iter);
       }
     }
   }
 
   /**
-   * no_io and const_ikey_ptr here means the same as in KeyMayMatch
+   * Similar to KeyMayMatch
    */
-  virtual bool PrefixMayMatch(const Slice& prefix, const bool no_io,
+  virtual bool PrefixMayMatch(const Slice& prefix,
                               const Slice* const const_ikey_ptr,
                               GetContext* get_context,
                               BlockCacheLookupContext* lookup_context,
@@ -140,7 +160,6 @@ class FilterBlockReader {
 
   virtual void PrefixesMayMatch(MultiGetRange* range,
                                 const SliceTransform* prefix_extractor,
-                                const bool no_io,
                                 BlockCacheLookupContext* lookup_context,
                                 const ReadOptions& read_options) {
     for (auto iter = range->begin(); iter != range->end(); ++iter) {
@@ -148,8 +167,8 @@ class FilterBlockReader {
       const Slice ikey = iter->ikey;
       GetContext* const get_context = iter->get_context;
       if (prefix_extractor->InDomain(ukey_without_ts) &&
-          !PrefixMayMatch(prefix_extractor->Transform(ukey_without_ts), no_io,
-                          &ikey, get_context, lookup_context, read_options)) {
+          !PrefixMayMatch(prefix_extractor->Transform(ukey_without_ts), &ikey,
+                          get_context, lookup_context, read_options)) {
         range->SkipKey(iter);
       }
     }
@@ -169,13 +188,15 @@ class FilterBlockReader {
     return Status::OK();
   }
 
+  virtual void EraseFromCacheBeforeDestruction(
+      uint32_t /*uncache_aggressiveness*/) {}
+
   virtual bool RangeMayExist(const Slice* /*iterate_upper_bound*/,
                              const Slice& user_key_without_ts,
                              const SliceTransform* prefix_extractor,
                              const Comparator* /*comparator*/,
                              const Slice* const const_ikey_ptr,
                              bool* filter_checked, bool need_upper_bound_check,
-                             bool no_io,
                              BlockCacheLookupContext* lookup_context,
                              const ReadOptions& read_options) = 0;
 };
